@@ -1,8 +1,10 @@
 import sys
+from functools import partial
 from .ec_handler import Handler
 from .ec_classes import RuntimeError
-from PySide6.QtCore import Qt, QTimer, Signal
-from PySide6.QtGui import QPixmap
+from .ec_border import Border
+from PySide6.QtCore import Qt, QTimer, Signal, QRect
+from PySide6.QtGui import QPixmap, QPainter
 from PySide6.QtWidgets import (
     QApplication,
     QCheckBox,
@@ -34,7 +36,8 @@ from PySide6.QtWidgets import (
     QSizePolicy,
     QDialog,
     QMessageBox,
-    QDialogButtonBox
+    QDialogButtonBox,
+    QGraphicsDropShadowEffect
 )
 
 class Graphics(Handler):
@@ -61,23 +64,12 @@ class Graphics(Handler):
             'lineinput',
             'multiline',
             'listbox',
-            'combobox'
+            'combobox',
+            'widget'
             ]
 
     def dialogTypes(self):
-        return ['confirm', 'lineedit', 'multiline']
-
-    class ECDialog(QDialog):
-        def __init__(self, parent, record):
-            super().__init__(parent)
-            self.record = record
-        
-        def showEvent(self, event):
-            super().showEvent(event)
-            QTimer.singleShot(100, self.afterShown)
-        
-        def afterShown(self):
-            if 'action' in self.record: self.record['action']()
+        return ['confirm', 'lineedit', 'multiline', 'generic']
 
     class ClickableLineEdit(QLineEdit):
         clicked = Signal()
@@ -274,7 +266,8 @@ class Graphics(Handler):
         return False
     
     def r_clear(self, command):
-        self.getVariable(command['name'])['widget'].clear()
+        widget = self.getVariable(command['name'])['widget']
+        widget.clear()
         return self.nextPC()
 
     # close {window}
@@ -300,7 +293,7 @@ class Graphics(Handler):
 
     # Create a window
     def k_createWindow(self, command):
-        command['title'] = 'Default'
+        title = None
         x = None
         y = None
         w = self.compileConstant(640)
@@ -309,7 +302,7 @@ class Graphics(Handler):
             token = self.peek()
             if token in ['title', 'at', 'size', 'layout']:
                 self.nextToken()
-                if token == 'title': command['title'] = self.nextValue()
+                if token == 'title': title = self.nextValue()
                 elif token == 'at':
                     x = self.nextValue()
                     y = self.nextValue()
@@ -323,6 +316,7 @@ class Graphics(Handler):
                             command['layout'] = record['name']
                 else: return False
             else: break
+        command['title'] = title
         command['x'] = x
         command['y'] = y
         command['w'] = w
@@ -427,11 +421,7 @@ class Graphics(Handler):
         self.add(command)
         return True
 
-    def k_createListWidget(self, command):
-        self.add(command)
-        return True
-
-    def k_createComboBox(self, command):
+    def k_createWidget(self, command):
         self.add(command)
         return True
 
@@ -456,6 +446,9 @@ class Graphics(Handler):
             elif self.peek() == 'value':
                 self.nextToken()
                 command['value'] =  self.nextValue()
+            elif self.peek() == 'with':
+                self.nextToken()
+                command['layout'] =  self.nextToken()
             else: break
         if not 'title' in command: command['title'] = self.compileConstant('')
         if not 'value' in command: command['value'] = self.compileConstant('')
@@ -495,6 +488,7 @@ class Graphics(Handler):
             command['name'] = record['name']
             keyword = record['keyword']
             if keyword == 'window': return self.k_createWindow(command)
+            elif keyword in ['listbox', 'combobox', 'widget']: return self.k_createWidget(command)
             elif keyword == 'layout': return self.k_createLayout(command)
             elif keyword == 'group': return self.k_createGroupBox(command)
             elif keyword == 'label': return self.k_createLabel(command)
@@ -502,15 +496,15 @@ class Graphics(Handler):
             elif keyword == 'checkbox': return self.k_createCheckBox(command)
             elif keyword == 'lineinput': return self.k_createLineEdit(command)
             elif keyword == 'multiline': return self.k_createMultiLineEdit(command)
-            elif keyword == 'listbox': return self.k_createListWidget(command)
-            elif keyword == 'combobox': return self.k_createComboBox(command)
             elif keyword == 'dialog': return self.k_createDialog(command)
             elif keyword == 'messagebox': return self.k_createMessageBox(command)
         return False
     
     def r_createWindow(self, command, record):
         window = QMainWindow()
-        window.setWindowTitle(self.getRuntimeValue(command['title']))
+        title = self.getRuntimeValue(command['title'])
+        if title == None: title = 'EasyCoder Main Window'
+        window.setWindowTitle(title)
         w = self.getRuntimeValue(command['w'])
         h = self.getRuntimeValue(command['h'])
         x = command['x']
@@ -574,6 +568,7 @@ class Graphics(Handler):
             c = pushbutton.contentsMargins()
             w = fm.horizontalAdvance('m') * self.getRuntimeValue(command['size']) + c.left()+c.right()
             pushbutton.setMaximumWidth(w)
+        self.putSymbolValue(record, pushbutton)
         record['widget'] = pushbutton
         return self.nextPC()
     
@@ -628,33 +623,66 @@ class Graphics(Handler):
         record['widget'] = QComboBox()
         return self.nextPC()
     
+    def r_createWidget(self, command, record):
+        record['widget'] = QWidget()
+        return self.nextPC()
+    
     def r_createDialog(self, command, record):
+
+        class ECDialog(QDialog):
+            def __init__(self, parent, record):
+                super().__init__(parent)
+                self.record = record
+            
+            def showEvent(self, event):
+                super().showEvent(event)
+                QTimer.singleShot(100, self.afterShown)
+            
+            def afterShown(self):
+                if 'action' in self.record: self.record['action']()
+
         win = command['window']
         if win != None:
             win = self.getVariable(win)['window']
-        dialog = self.ECDialog(win, record)
-        mainLayout = QVBoxLayout(dialog)
-        dialog.setWindowTitle(self.getRuntimeValue(command['title']))
+        dialog = ECDialog(win, record)
         dialogType = command['type'].lower()
         dialog.dialogType = dialogType
-        prompt = self.getRuntimeValue(command['prompt'])
-        if dialogType == 'confirm':
-            mainLayout.addWidget(QLabel(prompt))
-        elif dialogType == 'lineedit':
-            mainLayout.addWidget(QLabel(prompt))
-            dialog.lineEdit = self.ClickableLineEdit(dialog)
-            dialog.value = self.getRuntimeValue(command['value'])
-            dialog.lineEdit.setText(dialog.value)
-            mainLayout.addWidget(dialog.lineEdit)
-        elif dialogType == 'multiline':
-            mainLayout.addWidget(QLabel(prompt))
-            dialog.textEdit = self.ClickablePlainTextEdit(self)
-            dialog.textEdit.setText(dialog.value)
-            mainLayout.addWidget(dialog.textEdit)
-        buttonBox = QDialogButtonBox(QDialogButtonBox.Ok | QDialogButtonBox.Cancel)
-        buttonBox.accepted.connect(dialog.accept)
-        buttonBox.rejected.connect(dialog.reject)
-        mainLayout.addWidget(buttonBox, alignment=Qt.AlignHCenter)
+        mainLayout = QVBoxLayout(dialog)
+        if dialogType == 'generic':
+            dialog.setFixedWidth(500)
+            dialog.setFixedHeight(500)
+            dialog.setWindowFlags(Qt.FramelessWindowHint)
+            dialog.setModal(True)
+            dialog.setStyleSheet('background-color: white;border:1px solid black;')
+
+            border = Border()
+            border.tickClicked.connect(dialog.accept)
+            border.closeClicked.connect(dialog.reject)
+            mainLayout.addWidget(border)
+            if 'layout' in command:
+                layout = self.getVariable(command['layout'])['widget']
+                mainLayout.addLayout(layout)
+            dialog.setLayout(mainLayout)
+        else:
+            dialog.setWindowTitle(self.getRuntimeValue(command['title']))
+            prompt = self.getRuntimeValue(command['prompt'])
+            if dialogType == 'confirm':
+                mainLayout.addWidget(QLabel(prompt))
+            elif dialogType == 'lineedit':
+                mainLayout.addWidget(QLabel(prompt))
+                dialog.lineEdit = self.ClickableLineEdit(dialog)
+                dialog.value = self.getRuntimeValue(command['value'])
+                dialog.lineEdit.setText(dialog.value)
+                mainLayout.addWidget(dialog.lineEdit)
+            elif dialogType == 'multiline':
+                mainLayout.addWidget(QLabel(prompt))
+                dialog.textEdit = self.ClickablePlainTextEdit(self)
+                dialog.textEdit.setText(dialog.value)
+                mainLayout.addWidget(dialog.textEdit)
+            buttonBox = QDialogButtonBox(QDialogButtonBox.Ok | QDialogButtonBox.Cancel)
+            buttonBox.accepted.connect(dialog.accept)
+            buttonBox.rejected.connect(dialog.reject)
+            mainLayout.addWidget(buttonBox, alignment=Qt.AlignHCenter)
         record['dialog'] = dialog
         return self.nextPC()
     
@@ -681,6 +709,7 @@ class Graphics(Handler):
         elif keyword == 'multiline': return self.r_createMultiLineEdit(command, record)
         elif keyword == 'listbox': return self.r_createListWidget(command, record)
         elif keyword == 'combobox': return self.r_createComboBox(command, record)
+        elif keyword == 'widget': return self.r_createWidget(command, record)
         elif keyword == 'dialog': return self.r_createDialog(command, record)
         elif keyword == 'messagebox': return self.r_createMessageBox(command, record)
         return None
@@ -852,6 +881,10 @@ class Graphics(Handler):
         return False
     
     def r_on(self, command):
+        def run(widget):
+            print(f"Widget {widget} was clicked")
+            self.run(command['goto'])
+
         if command['type'] == 'tick':
             self.runOnTick = command['runOnTick']
         else:
@@ -859,7 +892,8 @@ class Graphics(Handler):
             widget = record['widget']
             keyword = record['keyword']
             if keyword == 'pushbutton':
-                widget.clicked.connect(lambda: self.run(command['goto']))
+                handler = partial(run, widget)
+                widget.clicked.connect(handler)
             elif keyword == 'combobox':
                 widget.currentIndexChanged.connect(lambda: self.run(command['goto']))
             elif keyword == 'listbox':
@@ -938,11 +972,12 @@ class Graphics(Handler):
         return self.nextPC()
 
     # set [the] width/height [of] {widget} [to] {value}
-    # set [the] layout of {window} to {layout}
+    # set [the] layout of {window}/{widget} to {layout}
     # set [the] spacing of {layout} to {value}
     # set [the] text [of] {label}/{button}/{lineinput}/{multiline} [to] {text}
     # set [the] color [of] {label}/{button}/{lineinput}/{multiline} [to] {color}
     # set [the] state [of] {checkbox} [to] {state}
+    # set [the] style of {widget} to {style}
     # set {listbox} to {list}
     # set blocked true/false
     def k_set(self, command):
@@ -963,7 +998,8 @@ class Graphics(Handler):
             self.skip('of')
             if self.nextIsSymbol():
                 record = self.getSymbolRecord()
-                if record['keyword'] == 'window':
+                keyword = record['keyword']
+                if keyword in ['window', 'widget']:
                     command['name'] = record['name']
                     self.skip('to')
                     if self.nextIsSymbol():
@@ -996,6 +1032,22 @@ class Graphics(Handler):
             if self.nextIsSymbol():
                 record = self.getSymbolRecord()
                 if record['keyword'] == 'checkbox':
+                    command['name'] = record['name']
+                    self.skip('to')
+                    if self.peek() == 'checked':
+                        command['value'] = self.compileConstant(True)
+                        self.nextToken()
+                    elif self.peek() == 'unchecked':
+                        command['value'] = self.compileConstant(False)
+                        self.nextToken()
+                    else: command['value'] = self.nextValue()
+                    self.add(command)
+                    return True
+        elif token == 'style':
+            self.skip('of')
+            if self.nextIsSymbol():
+                record = self.getSymbolRecord()
+                if record['extra'] == 'gui':
                     command['name'] = record['name']
                     self.skip('to')
                     command['value'] = self.nextValue()
@@ -1068,16 +1120,21 @@ class Graphics(Handler):
             widget = self.getVariable(command['name'])['widget']
             widget.setFixedWidth(self.getRuntimeValue(command['value']))
         elif what == 'layout':
-            window = self.getVariable(command['name'])['window']
             content = self.getVariable(command['layout'])['widget']
-            container = QWidget()
-            container.setLayout(content)
-            window.setCentralWidget(container)
+            record = self.getVariable(command['name'])
+            keyword = record['keyword']
+            if keyword == 'window':
+                window = record['widget']
+                container = QWidget()
+                container.setLayout(content)
+                window.setCentralWidget(container)
+            elif keyword == 'widget':
+                widget = record['widget']
+                widget.setLayout(content)
         elif what == 'spacing':
             layout = self.getVariable(command['name'])['widget']
             layout.setSpacing(self.getRuntimeValue(command['value']))
         elif what == 'text':
-            record = self.getVariable(command['name'])
             widget = self.getVariable(command['name'])['widget']
             text = self.getRuntimeValue(command['value'])
             keyword = record['keyword']
@@ -1188,7 +1245,9 @@ class Graphics(Handler):
         elif 'dialog' in command:
             record = self.getVariable(command['dialog'])
             dialog = record['dialog']
-            if dialog.dialogType == 'confirm':
+            if dialog.dialogType == 'generic':
+                record['result'] =  dialog.exec()
+            elif dialog.dialogType == 'confirm':
                 record['result'] = True if dialog.exec() == QDialog.Accepted else False
             elif dialog.dialogType == 'lineedit':
                 if dialog.exec() == QDialog.Accepted:
@@ -1224,6 +1283,13 @@ class Graphics(Handler):
         self.app.lastWindowClosed.connect(on_last_window_closed)
         self.app.exec()
 
+    # Declare a widget variable
+    def k_widget(self, command):
+        return self.compileVariable(command, 'gui')
+
+    def r_widget(self, command):
+        return self.nextPC()
+
     # Declare a window variable
     def k_window(self, command):
         return self.compileVariable(command)
